/*
* Copyright c                  Realtek Semiconductor Corporation, 2003
* All rights reserved.
* 
* Program : Internal Header file for  fowrading engine rtl8651_tblDrvFwd.c
* Abstract : 
* Author :  Chun-Feng Liu(cfliu@realtek.com.tw)
* $Id: rtl8651_tblDrvFwdLocal.h,v 1.6 2004/11/10 07:40:18 chenyl Exp $
* $Log: rtl8651_tblDrvFwdLocal.h,v $
* Revision 1.6  2004/11/10 07:40:18  chenyl
* *: fragment cache re-write (1. code re-arrange, 2. check org-sip/dip when getting cache)
*
* Revision 1.5  2004/10/04 02:16:23  chenyl
* +: pptp/l2tp header refill function
*
* Revision 1.4  2004/09/30 07:59:12  chenyl
* *: function mtuFragment() modification
* +: apply forwarding engine to generate ICMP-Host-Unreachable message for packets fragmented by gateway
* +: add the switch to turn on/off "always reply ICMP error message for all packets fragmented by gateway regardless of
* 	Dont fragment bit"
*
* Revision 1.3  2004/09/15 16:14:28  chenyl
* +: enable multicast forward cache
*
* Revision 1.2  2004/09/13 04:17:36  chenyl
* +: napt redirect inbound fragment cache
*
* Revision 1.1  2004/09/07 14:52:13  chenyl
* *: bug fix: napt-redirect fragment packet checksum-recalculate
* *: bug fix: conflict between protocol stack flow cache and protocol stack action
* *: bug fix: protocol stack action mis-process to UDP-zero-checksum packet
* *: separate the header file:
*         - internal : rtl8651_tblDrvFwdLocal.h
*         - external : rtl8651_tblDrvFwd.h
*
*/
#ifndef RTL8651_FWDENGINE_LOCAL_H
#define RTL8651_FWDENGINE_LOCAL_H

#include "assert.h"
#include "types.h"
#ifndef CYGWIN
#include <linux/config.h>
#endif /* CYGWIN */
#include "rtl_queue.h"
#include "rtl_errno.h"
#include "mbuf.h"

#include "rtl8651_tblDrvProto.h"
#include "rtl8651_tblDrvLocal.h"
#include "rtl8651_tblDrvFwd.h"
#include "rtl8651_ipQueue_local.h"
#include "rtl8651_multicast_local.h"

/*********************************************************************************************************
	ACL
**********************************************************************************************************/
int32 _rtl8651_ingressAcl(uint32 dsid,rtl8651_tblDrv_vlanTable_t *pVlan,int32 vid, uint8 *m_data, uint8 *ip);
int32 _rtl8651_egressAcl(uint32 dsid,rtl8651_tblDrv_vlanTable_t *pVlan,int32 vid, uint8 *ethHdr, uint8 *ip);

/*********************************************************************************************************
	L2 forward
**********************************************************************************************************/
int32 rtl8651_fwdEngineL2Input(struct rtl_pktHdr *pkthdrPtr,rtl8651_tblDrv_vlanTable_t *local_vlanp, uint8 *m_data, uint16 ethtype);

/*********************************************************************************************************
	ALG
**********************************************************************************************************/
void _rtl8651_l4AddSeq(struct ip *, rtl8651_tblDrv_naptTcpUdpFlowEntry_t *, int32 );
int32 _rtl8651_l4SeqDiff(uint32 , uint32 );
int32 _rtl8651_l4GetDeltaSeqOut(struct ip *, struct tcphdr *, rtl8651_tblDrv_naptTcpUdpFlowEntry_t  *);

/*********************************************************************************************************
	Fragment Shortest Path Forwarding
**********************************************************************************************************/
#define RTL8651_FRAGMENT_SHORTESTPATH_FWD
#ifdef RTL8651_FRAGMENT_SHORTESTPATH_FWD

#define RTL8651_FRAGMENT_SHORTESTPATH_STATUS_HALFOK	1
#define RTL8651_FRAGMENT_SHORTESTPATH_STATUS_SETOK	2

#define RTL8651_FRAGMENT_SHORTESTPATH_MODE_MODIFYSIP	0
#define RTL8651_FRAGMENT_SHORTESTPATH_MODE_MODIFYDIP	1

#define RTL8651_FRAGMENT_SHORTESTPATH_TIMEOUT			3

typedef struct _rtl8651_FragmentShortestPathFwdCache_s
{
	/* information to search flow entry */
	uint8 status;
	ipaddr_t sip;
	ipaddr_t dip;
	uint16 id;
	/* information to forward matched packet */
	uint8 mode;
	ipaddr_t aliasIp;
//	rtl8651_tblDrv_vlanTable_t *dst_vlan;
	struct ether_addr smac;
	struct ether_addr dmac;
	uint16 vlanIdx;
	uint16 portList;
	uint8 extPortList;
	uint8 srcExtPortNum;
	uint16 pppoeIdx;
	uint16 flags;		/* checksum & pppoe_auto_add */
	/* timeout */
	uint8 age;
} _rtl8651_FragmentShortestPathFwdCache_t;

#define RTL8651_FRAGMENTSHORTESTPATH_TBLSIZE	32	/* must be power-of-2 */

inline void _rtl8651_fragmentShortestPathCacheSet_secondHalf(struct rtl_pktHdr *pkthdr, uint8 *m_data, struct ip*pip);
extern _rtl8651_FragmentShortestPathFwdCache_t fragmentShortestPathCache[RTL8651_FRAGMENTSHORTESTPATH_TBLSIZE];
#endif /* RTL8651_FRAGMENT_SHORTESTPATH_FWD */

/*********************************************************************************************************
	ICMP reply
**********************************************************************************************************/
// define parameters passed to fwd-engine ICMP reply functions
typedef struct _rtl8651_IcmpPktField_s{
	uint8	ic_type;
	uint8	ic_code;
	union{
		uint16	next_mtu;
	} ic_hun;
	#define ic_mtu	ic_hun.next_mtu
} _rtl8651_IcmpPktField_t;
int32 _rtl8651_drvIcmpErrorGeneration(struct rtl_pktHdr *, struct ip*, _rtl8651_IcmpPktField_t *, uint16);

/*********************************************************************************************************
	UPnP
**********************************************************************************************************/
#ifdef _RTL_UPNP
struct _RTL8651_UPNP_ENTRY{
	ipaddr_t remote_ip;
	uint16 remote_port; 
	ipaddr_t alias_ip;
	uint16 alias_port;
	ipaddr_t local_ip;
	uint16 local_port;	
	
	uint32 flags;
	uint32 age; /* second */
};
#define UPNP_VLAID		0x01
#define UPNP_TCP		0x02
#define UPNP_ONESHOT	0x04
#define UPNP_PERSIST	0x08

#define UPNP_ITEM 60
#define UPNP_AGE 120
#endif /* _RTL_UPNP */

/*********************************************************************************************************
	Trigger port
**********************************************************************************************************/
#ifdef _RTL_TRIGGER_PORT
#define TRIGGER_PORT_ITEM 64
struct _RTL8651_TRIGGER_PORT_ENTRY{
	uint32 dsid;
	ipaddr_t localIpAddr;
	uint8  inType;
	uint8  outType;
	uint16 inPortStart; 
	uint16 inPortFinish; 
	uint16 outPortStart; 
	uint16 outPortFinish;
};
#endif /* _RTL_TRIGGER_PORT */

/*********************************************************************************************************
	Port bouncing
**********************************************************************************************************/
#ifdef _RTL_PORT_BOUNCING
#define PORT_BOUNCING_ITEM 64
struct _RTL8651_PORT_BOUNCING_ENTRY{
	
	ipaddr_t clientIpAddr;
	ipaddr_t gatewayIpAddr;	
	ipaddr_t serverIpAddr;
	uint16 clientPort; 
	uint16 gatewayPort; 	
	uint16 serverPort;
	uint16 id;					// for ICMP flow: l4 id field
	uint16 flowType;
	uint32 tcpFlag;				// state for tcp flow
	uint32 age;
};
#endif /* _RTL_PORT_BOUNCING */

/*********************************************************************************************************
	Fragment cache
**********************************************************************************************************/
#ifdef _RTL_NAPT_FRAG
#define DEFAULT_MAX_FRAGPKT_NUM				32
#define DEFAULT_MAX_FRAGCACHE_AGE			5
#define INBOUND_FRAG							0
#define OUTBOUND_FRAG							1
#define BOUNCING_FRAG							2
#define ICMP_REPLY_FRAG						3
#define PROTOCOL_STACK_FRAG					4
#define NAPT_REDIRECT_FRAG						5
#define NAPT_REDIRECT_FRAG_INBOUND			6
#define RTL8651_FRAGCACHE_TYPE_COUNT			7	/* max count of fragment cache */
typedef struct _rtl8651_frag {
	/* <--- Informations ---> */
#if 0
	union{
		uint32	local_ip_addr;
		uint32	alias_ip_addr;
	} u;
#else
	ipaddr_t		new_ip;
#endif	/* 0 */
	int8			routeIdx;
	void 		*napt_entry_t;
	/* <--- Control Informations ---> */
	uint8		age;
	/* <--- Index ---> */
	uint16		ip_id;
	ipaddr_t		org_sip;
	ipaddr_t		org_dip;
} _rtl8651_frag_t;
#endif /* _RTL_NAPT_FRAG */

/*********************************************************************************************************
	Pppoe active session table
**********************************************************************************************************/
typedef struct _rtl8651_pppoeActiveSession_entry_s {
	uint16 sessionId;
	uint16 keepAlive;//When value down-count to zero, this entry becomes invalid
	struct _rtl8651_pppoeActiveSession_entry_s * next;
} _rtl8651_pppoeActiveSession_entry_t;

/*********************************************************************************************************
	mbuf related
**********************************************************************************************************/
#define FWDENG_MBUF_HEADROOM	128  /*default headroom reserved*/

/*********************************************************************************************************
	forwarding engine initiation
**********************************************************************************************************/
extern rtl8651_fwdEngineInitPara_t rtl8651_fwdEnginePara;
#define FWD_INIT_CHECK(expr) do {\
	if((expr)!=SUCCESS){\
		rtlglue_printf("Error >>> initialize failed at line %d!!!\n", __LINE__);\
			return FAILED;\
	}\
}while(0)

/*********************************************************************************************************
	protocol stack action
**********************************************************************************************************/
//#define RTL8651_MAX_PROTO_STACK_USED_UDP_PORTS 8
//#define RTL8651_MAX_PROTO_STACK_SERVER_USED_TCP_PORTS 8

/*********************************************************************************************************
	Checksum
**********************************************************************************************************/
uint16  _rtl8651_ipChecksum(struct ip * pip);
uint16 _rtl8651_icmpChecksum(struct ip * pip);
uint16 _rtl8651_tcpChecksum(struct ip * pip);	

/* Checksum adjustment */
#define	_RTL8651_ADJUST_CHECKSUM(acc, cksum) \
	do { \
		acc += ntohs(cksum); \
		if (acc < 0) { \
			acc = -acc; \
			acc = (acc >> 16) + (acc & 0xffff); \
			acc += acc >> 16; \
			cksum = htons((uint16) ~acc); \
		} else { \
			acc = (acc >> 16) + (acc & 0xffff); \
			acc += acc >> 16; \
			cksum = htons((uint16) acc); \
		} \
	} while (0)

#define _RTL8651_DATA_ADJUST_CHECKSUM(data_mod, data_org, cksum) \
	do { \
		int32 accumulate; \
		accumulate = ((data_org) - (data_mod)); \
		_RTL8651_ADJUST_CHECKSUM( accumulate, (cksum)); \
	} while(0)
	
#define _RTL8651_L4_ADJUST_CHECKSUM(ip_mod, ip_org, port_mod, port_org, cksum) \
	do { \
		int32 accumulate = 0; \
		if (((ip_mod) != 0) && ((ip_org) != 0)){ \
			accumulate = ((ip_org) & 0xffff); \
			accumulate += (( (ip_org) >> 16 ) & 0xffff); \
			accumulate -= ((ip_mod) & 0xffff); \
			accumulate -= (( (ip_mod) >> 16 ) & 0xffff); \
		} \
		if (((port_mod) != 0) && ((port_org) != 0)){ \
			accumulate += (port_org); \
			accumulate -= (port_mod); \
		} \
		_RTL8651_ADJUST_CHECKSUM(accumulate, (cksum)); \
	}while(0)

#define _RTL8651_L3_ADJUST_CHECKSUM(ip_mod, ip_org, cksum) \
	do {\
		_RTL8651_L4_ADJUST_CHECKSUM((ip_mod), (ip_org), 0, 0, (cksum)); \
	}while(0)

/*********************************************************************************************************
	GRE/PPTP
**********************************************************************************************************/
#define hton8(x)  (x)
#define ntoh8(x)  (x)
#define hton16(x) htons(x)
#define ntoh16(x) ntohs(x)
#define hton32(x) htonl(x)
#define ntoh32(x) ntohl(x)

#if 0
#define PPTP_GRE_PROTO  0x880B
#define PPTP_GRE_VER    0x1

#define PPTP_GRE_FLAG_C	0x80
#define PPTP_GRE_FLAG_R	0x40
#define PPTP_GRE_FLAG_K	0x20
#define PPTP_GRE_FLAG_S	0x10
#define PPTP_GRE_FLAG_A	0x80

#define PPTP_GRE_IS_C(f) ((f)& PPTP_GRE_FLAG_C)
#define PPTP_GRE_IS_R(f) ((f)& PPTP_GRE_FLAG_R)
#define PPTP_GRE_IS_K(f) ((f)& PPTP_GRE_FLAG_K)
#define PPTP_GRE_IS_S(f) ((f)& PPTP_GRE_FLAG_S)
#define PPTP_GRE_IS_A(f) ((f)& PPTP_GRE_FLAG_A)
#endif

struct wan_gre_header {
  uint8 flags;			/* bitfield */
  uint8 ver;			/* should be PPTP_GRE_VER (enhanced GRE) */
  uint16 protocol;		/* should be PPTP_GRE_PROTO (ppp-encaps) */
  uint16 payload_len;	/* size of ppp payload, not inc. gre header */
  uint16 call_id;		/* peer's call_id for this session */
  uint32 seq;			/* sequence number.  Present if S==1 */
  uint32 ack;			/* seq number of highest packet recieved by */
  						/*  sender in this session */
};
#if 0
struct _RTL8651_PPTP_SHARE_DATA {
		uint32 seq;
		uint32 ack; 
		uint32 seq_recv;	
		uint32 seq_sent;
		uint32 ack_sent;
		uint32 ack_recv;
		
		uint32 tunnel_dst_ip;
		uint32 tunnel_src_ip;
		
		uint16 pptp_gre_call_id;
		uint16 pptp_gre_peer_call_id;
		
		uint8 reserv[128];
};
#endif

/*********************************************************************************************************
	L2TP
**********************************************************************************************************/
struct wan_l2tp_header {

  uint16 flags_type:1;
  uint16 flags_length:1;
  uint16 flags_rsv1:2;
  uint16 flags_sequence:1;
  uint16 flags_rsv2:1;
  uint16 flags_offset:1;
  uint16 flags_priority:1;  
  uint16 flags_rsv3:4;  
  uint16 flags_version:4;
  
  uint16 length;
  uint16 tunnel_id;
  uint16 session_id;  
};

/*********************************************************************************************************
	Port Scan
**********************************************************************************************************/
#if 0
//for ACK Scan, SYN Scan, FIN Scan
#define PORT_SCAN_WAIT_RST 2
struct portScanInfo {
	ipaddr_t sip;
	ipaddr_t dip;
	uint32 dsid;
	uint16 sport;
	uint16 dport;
	uint8 flags;
	uint8 age; 
};
#endif

/*********************************************************************************************************
	PPTP-Header Cache
**********************************************************************************************************/
#ifdef _RTL_PPTPL2TP_HDR_REFILL
typedef struct _rtl8651_pptpHdrCacheEntry_s
{
	/* <-- data to be cached --> */
	uint8 type;
	uint32 seq;
	uint32 ack;
	/* <-- key - we only support ip-in-GRE type, so we can store id/offset in ip-header as the key to be hashed --> */
	uint16 id;
	uint16 offset;
	/* <-- for linked list -->*/
	struct _rtl8651_pptpHdrCacheEntry_s **hdr;
	struct _rtl8651_pptpHdrCacheEntry_s *prev;
	struct _rtl8651_pptpHdrCacheEntry_s *next;
	struct _rtl8651_pptpHdrCacheEntry_s *lru_prev;
	struct _rtl8651_pptpHdrCacheEntry_s *lru_next;
} _rtl8651_pptpHdrCacheEntry_t;

#define RTL8651_PPTPHDR_SEQ_SET	0x01
#define RTL8651_PPTPHDR_ACK_SET	0x02

#define RTL8651_PPTPHDR_CACHE_HASH_TBLSIZE	64
#define RTL8651_PPTPHDR_CACHE_DEFAULT_ENTRYCNT	192
/* easy hash --> hash table size MUST be power of 2 */
#define RTL8651_IP_ID_EASYHASH(sip, dip, id, hashTblSize)	(((uint32)(sip)+(uint32)(dip)+(uint32)(htons(id))) & (hashTblSize-1))

void _rtl8651_pptpHdrCache_Add(uint16 ip_id, uint16 ip_off, uint32 seq, uint32 ack, uint8 type);
#endif /* _RTL_PPTPL2TP_HDR_REFILL */
/*********************************************************************************************************
	mtu
**********************************************************************************************************/
int32 _rtl8651_mtuFragment(struct rtl_pktHdr *pkthdrPtr, uint32 mtu, uint16 dvid, int8 isNapt, uint16 srcPortList);

/*********************************************************************************************************
	misc
**********************************************************************************************************/
/* fragment check */
#define DONT_FRAG(off)				(ntohs((uint16)(off)) & (IP_DF))
#define IS_FRAGMENT(off)				(ntohs((uint16)(off)) & (IP_OFFMASK|IP_MF))
#define IS_FRAGMENT_FIRST(off)		((ntohs((uint16)(off)) & (IP_OFFMASK|IP_MF)) == IP_MF)
#define IS_FRAGMENT_REMAIN(off)	(ntohs((uint16)(off)) & IP_OFFMASK)
/* ip header length related */
#define IPHDRLEN_IS_TWENTY(iphdr)		(((*(uint8*)iphdr)&0xf) == 5)					/* check IP header Length is 20 or not */
#define IPHDR_LEN(iphdr)					(((*(uint8*)iphdr)&0xf) << 2)					/* get the ip length */
#define L4HDR_ADDR(iphdr)				((int8 *)iphdr + (((*(uint8*)iphdr)&0xf) << 2))	/* calculate L4 header address by ip header */
#define _RTL8651_TTL_DEC_1(ip_ttl, ip_sum) \
	do { \
		int32 ttl_mod = 0x0100; \
		(ip_ttl) --; \
		_RTL8651_ADJUST_CHECKSUM(ttl_mod, (ip_sum)); \
	} while(0)


/* extern variables */
extern int8 rtl8651_drvMulticastProcessEnable;
extern int8 rtl8651_drvPacketFastSend;
extern int8 rtl8651_drvIcmpRoutingMsgEnable;
extern rtl8651_tblDrvFwdEngineCounts_t rtl8651_fwdEngineCounter;


#endif /* RTL8651_FWDENGINE_LOCAL_H */
